/*
	Automatically generated source file template for a task.
	(c) European Space Agency


	Title: dsnullify
	Package: daltools
	Author: Mark Thomas
	Affiliation: CARA/ESA
	Email: mthomas@astro.estec.esa.nl
	Purpose: Set all data elements in an object to null
	Description: Set all data elements in an object to null


*/

#include <Tracer.h>
#include <errstr.h>
#include <param.h>
#include <Dal.h>
#include <CompoundName.h>


static void
nullify( Data* data, unsigned long elements )
{
	PROC
	for( unsigned int i =  0; i < elements; ++i )
	{
		data -> setToNull( i );
	} // for
	ENDPROC
}


static void
nullify( Column* column )
{
	PROC
	if( column -> cellType() == Column::Fixed )
	{
		ColumnData*		cd = column -> data();
		const unsigned long	elements = column -> rows() * column -> elements();

		nullify( cd, elements );
		delete cd;
	}
	else
	{
		for( unsigned long i = 0; i < column -> rows(); ++i )
		{
			CellData*		cd = column -> cellData( i );
			const unsigned long	elements = cd -> size();

			nullify( cd, elements );
			delete cd;
		} // for
	} // if
	ENDPROC
}



static void
nullify( Table* table )
{
	PROC
	for( unsigned int i = 0; i < table -> columns(); ++i )
	{
		nullify( table -> column( i ) );
	} // for
	ENDPROC
}


static void
nullify( Array* array )
{
	PROC
	ArrayData* ad = array -> data();
	const unsigned long	elements = ad -> elements();
	nullify( ad, elements );
	delete ad;
	ENDPROC
}


static void
nullify( Block* block )
{
	PROC
	if( block -> type() == Block::ArrayT )
	{
		nullify( block -> parent() -> array( block -> name() ));
	}
	else
	{
		nullify( block -> parent() -> table( block -> name() ));
	} // if

	ENDPROC
}


static void
nullify( DataSet* set )
{
	PROC
	for( unsigned int i = 0; i < set -> blocks(); ++i )
	{
		nullify( set -> block( i ) );
	} // for
	ENDPROC
}


#include "dsnullify.h"

void dsnullify()
{
	PROC
	const vector<string>	objects( stringListParameter( "objects" ) );

	for( unsigned int i = 0; i < objects.size(); ++i )
	{
		const string		name( objects[ i ] );
		DataSet*		set( dataSetServer -> open( name, Dal::Modify, DataSetServer::Low ) );
		const CompoundName	cn( name, CompoundName::Any, set );

		switch( cn.state() )
		{
			case CompoundName::DataSetName :
				nullify( set );
				set = 0;
				break;
			case CompoundName::BlockName :
			case CompoundName::TableName :
			case CompoundName::ArrayName :
				nullify( set -> block( name ));
				break;
			case CompoundName::ColumnName :
				nullify( set -> table( name ) -> column( name ));
				break;
			default :
				errstr << "Object type " << name << " not supported" << Sas::error( "unsupportedObjectType" );
		} // switch

		if( set )
		{
			dataSetServer -> close( set );
		} // if
	} // for
	ENDPROC
}
