/*
	Automatically generated source file template for a task.
	(c) European Space Agency


	Title: dssetarrayelement
	Package: daltools
	Author: Mark Thomas
	Affiliation: CARA/ESA
	Email: mthomas@astro.estec.esa.nl
	Purpose: Set the value of an array element
	Description: Set the value of an array element.


*/

#include <Tracer.h>
#include <param.h>
#include <Dal.h>
#include <dal_utilities.h>

#include "dssetarrayelement.h"


template<class T>
static void
setElements(
	ArrayData*	ad,
	T* (Data::* data)() const,
	unsigned long	fromelement,
	unsigned long	elements,
	T		v )
{
	PROC

	T*	d;
	Data*	dd = ad;
	d = (dd ->*data)();
	//d = (cd ->*data)();

	for( unsigned long i = 0; i < numberOfElements( ad -> count() ); ++i )
	{
		for( unsigned long j = 0; j < elements; ++j )
		{
			d[ ad -> elements() * i + fromelement + j ] = v;
		} // for
	} // for

	ENDPROC
}


class Position
{
	public :
		Position( const longVec& coordinates ) : _c( coordinates ) {}
		const longVec& coordinates() const { PROC RETURN _c; }
		longVec from( unsigned long p ) const { PROC
			longVec	v;
			for( unsigned int i = 0; i < _c.size(); ++i )
			{
				unsigned long	d = 1;
				if( i > 0 )
				{
					for( unsigned long j = 0; j < i; ++j )
					{
						d += _c[ j ];
					} // for
				} // if
				v.push_back( ( p / d )% _c[ i ] );
			} // for

			//v.push_back( p % _c[ 0 ] );
			//v.push_back( ( p / _c[ 0 ] ) % _c[ 1 ] );
			//v.push_back( p / ( _c[ 0 ] * _c[ 1 ] ) % _c[ 2 ] );
			RETURN v;
		}
		longVec count( unsigned long p ) const { PROC
			longVec	v;
			for( unsigned int i = 0; i < _c.size(); ++i )
			{
				unsigned long	d = 1;
				if( i > 0 )
				{
					for( unsigned long j = 0; j < i; ++j )
					{
						d += _c[ j ];
					} // for
				} // if
				v.push_back( ( p / d )% _c[ i ] + 1 );
			} // for

			RETURN v;
		}
	private :
		const longVec _c;


};


void dssetarrayelement()
{
	PROC

	const vector<string>	arrays( stringListParameter( "arrays" ));
	const unsigned long	fromelement( intParameter( "fromelement" ));
	const string		value( stringParameter( "value" ));

	for( unsigned int i = 0; i < arrays.size(); ++i )
	{
		const string		arrayName( arrays[ i ] );
		DataSet*		set( dataSetServer -> open( arrayName, Dal::Modify, DataSetServer::Low ) );
		Array*			array( set -> array( arrayName ) );

		const unsigned long	elements( intParameter( "elements" ) ?  intParameter( "elements" ) : array -> elements() - fromelement );
		ArrayData*		ad( array-> data(
						Position( array -> size() ).from( fromelement),
						Position( array -> size()).count( elements ) ));

		switch( array -> dataType() )
		{
			case Array::Int8 :
				setElements(
					ad,
					&Data::int8Data,
					fromelement,
					elements,
					int8( atoi( value.c_str() )));
				break;
			case Array::Uint16 :
				setElements(
					ad,
					&Data::uint16Data,
					fromelement,
					elements,
					uint16( atoi( value.c_str() )));
				break;
			case Array::Int16 :
				setElements(
					ad,
					&Data::int16Data,
					fromelement,
					elements,
					int16( atoi( value.c_str() )));
				break;
			case Array::Uint32 :
				setElements(
					ad,
					&Data::uint32Data,
					fromelement,
					elements,
					uint32( atoi( value.c_str() )));
				break;
			case Array::Int32 :
				setElements(
					ad,
					&Data::int32Data,
					fromelement,
					elements,
					atoi( value.c_str() ));
				break;
			case Array::Real32 :
				setElements(
					ad,
					&Data::real32Data,
					fromelement,
					elements,
					real32( atof( value.c_str() )));
				break;
			case Array::Real64 :
				setElements(
					ad,
					&Data::real64Data,
					fromelement,
					elements,
					atof( value.c_str() ));
				break;
			default :
				errstr << "The array with qualified name '" << array -> qualifiedName() << "' has datatype '" << toString( array -> dataType() ) << "' which could not be handled" << Sas::error( "unhandledDataType" );
		} // switch

		delete ad;

		dataSetServer -> close( set );
	} // for

	ENDPROC
}
